# 第三方库
import pickle, re, requests, time
import pandas as pd
import numpy as np
from copy import deepcopy


class 设置相似文章():
    文章库 = {}
    推荐数量 = 5  # 每篇文章需要推荐的数量
    相似度缓存路径 = ''
    获取所有文章接口 = ''
    设置相似文章接口 = ""
    
    def __init__(self):
        self.检测数据完整性()  # 0
        self.创建文章库()  # 1
        self.载入相似度缓存()  # 1
        self.矫正相似度缓存字段()  # 2
    
    def 相似度分析(self, t1, t2):
        # return 0.856
        return '占位函数'
    
    def 检测数据完整性(self):
        # 判断相似度缓存路径
        if not self.相似度缓存路径: raise Exception("子类中需覆盖重写 相似度缓存路径, 且每个域名应设置不同的路径")
        if not self.获取所有文章接口: raise Exception("子类中需覆盖重写 获取所有文章接口")
        if not self.设置相似文章接口: raise Exception("子类中需覆盖重写 设置相似文章接口")
        if self.相似度分析(t1='aaaaa', t2='bbbbb') == '占位函数':
            raise Exception("子类中需覆盖重写 相似度分析 方法, 传入(t1, t2), return 浮点数")
    
    # 对 推荐文章为空 的文章进行推荐
    def 增量推荐文章(self):
        for 分类, 文章s in self.分类文章库.items():
            pks = [pk for pk, kw in 文章s.items() if not kw['相似文章pks']]
            if pks: self.pks_set_相似文章pks(分类, pks)
        print("增量推荐文章 完成")
    
    # 对 所有文章 进行推荐
    def 全部文章推荐文章(self):
        i, total = 0, len(self.分类文章库)
        for 分类, 文章s in self.分类文章库.items():
            i += 1
            print(f"\r                                            文章分类:{i}/{total}    ", end='')
            pks = list(文章s.keys())
            self.pks_set_相似文章pks(分类, pks)
        print("\n全部文章推荐文章 完成")
    
    # self.文章库 = {pk1:{kw1}, pk2:{kw2}, ...}
    # kw1 = [{pk:pk,标题:标题,相似文章pks:[1,2,3, ...]}]
    def 创建文章库(self):
        texts = requests.get(self.获取所有文章接口).json()
        文章库, 分类文章库 = {}, {}
        for pk, title, pks, sort in texts:
            pks = pks.replace(' ', '') if pks else []
            pks = list(map(int, pks.split('-'))) if pks else []
            sort = str(sort)
            kw = dict(pk=pk, title=title, 相似文章pks=pks, sort=sort)
            文章库[pk] = kw
            分类文章库.setdefault(sort, {})[pk] = kw
        self.文章库 = 文章库
        self.分类文章库 = 分类文章库
    
    # return 用来计算相似度的文本
    # 目前以title计算相似度, 后期可改为以正文计算相似度
    def pk_get_计算文本(self, pk):  return self.文章库[pk]['title']

    def 载入相似度缓存(self):
        try:
            with open(self.相似度缓存路径, 'rb') as fa:
                self.相似度缓存 = pickle.load(fa)
        except:
            self.相似度缓存 = {}

    # 由于文章增删变化, 需要修改缓存的列数
    def 矫正相似度缓存字段(self):
        # pks: {pk1, pk2, ...}
        for 分类, kws in self.分类文章库.items():
            df = self.相似度缓存.get(分类)
            if not (type(df) is pd.core.frame.DataFrame):
                self.相似度缓存[分类] = df = pd.DataFrame(dict(pk=[])).set_index('pk')
                df.index = df.index.astype(np.int64)
            pks = set(kws)
            旧的缓存字段 = set(df.columns)
            已失效pk = 旧的缓存字段 - pks
            新增的pk = pks - 旧的缓存字段
            已失效index = set(df.index) - pks
            if 已失效pk: df.drop(已失效pk, axis=1, inplace=True)
            if 已失效index: df.drop(已失效index, axis=0, inplace=True)
            for pk in 新增的pk: df[pk] = -1.0
        print('已矫正相似度缓存字段')
    
    def 保存相似度缓存(self):
        with open(self.相似度缓存路径, 'wb') as fa:
            pickle.dump(self.相似度缓存, fa)
        self.待保存计数 = 0

    # return {pk1:相似度1, pk2:相似度2, ...}
    待保存计数 = 0
    def pk_计算_相似的pks(self, 分类, pk, 推荐数量):
        df = self.相似度缓存[分类]
        if pk not in df.index: df.loc[pk] = -1.0
        相似度s = df.loc[pk]
        需计算的pk = 相似度s[相似度s < 0].index.to_list()
        if 需计算的pk:
            text = self.pk_get_计算文本(pk=pk)
            相似度s = deepcopy(相似度s)
            total = 相似度s.shape[0]
            for count,k in enumerate(需计算的pk):
                for i in range(10):
                    t2 = self.pk_get_计算文本(pk=k)
                    v = self.相似度分析(t1=text, t2=t2)
                    if (type(v) is float) or (type(v) is int):
                        相似度s[k] = v
                        break
                else:
                    相似度s[k] = v = 0
                print(f"\r{count}/{total}[{v}]        ", end='')
                self.待保存计数 += 1
                if self.待保存计数 >= 100: self.保存相似度缓存()
            df.loc[pk] = 相似度s = 相似度s.sort_values(ascending=False)
        return dict(相似度s[相似度s < 1][:推荐数量])
    
    # 对传入的pks设置推荐文章, 并提交到服务器
    def pks_set_相似文章pks(self, 分类, pks):
        if pks:
            文章库 = self.文章库
            total = len(pks)
            for i, pk in enumerate(pks):
                print(f"\r                         分类下的文章:{i}/{total}    ", end='')
                rpks = self.pk_计算_相似的pks(分类=分类, pk=pk, 推荐数量=self.推荐数量)
                文章库[pk]['相似文章pks'] = rpks = list(rpks.keys())
                rpks = '-'.join(map(str, rpks)) if rpks else ''
                url = self.设置相似文章接口.format(pk=pk, rpks=rpks)
                rt = requests.get(url).text  # ['设置成功', ]
            self.保存相似度缓存()



# -!- coding: utf-8 -!-
import datetime, re, time, hashlib
from math import ceil
from pathlib import Path



class tool():

    class date():
        
        def __init__(self, date): self.date = self.o_get_date(date=date)
        
        def __str__(self): return str(self.date)
        
        def __add__(self, count): return self.date + datetime.timedelta(days=int(count))
        
        o_get_date = lambda self, date: datetime.date(*map(int, re.findall('\d+', str(date))))
        
        def __sub__(self, date):
            try:
                return self.date - datetime.timedelta(days=int(str(date)))
            except:
                date = self.o_get_date(date=date)
                return 0 if self.date == date else int(str(self.date-date).split(' ')[0])

    def 数据分组(data, size):
        return [data[size*(i-1): size*i] for i in range(1, ceil(len(data)/size)+1)]
    
    def 提取任务名称(func):
        name = str(func)
        name = re.findall('function (.+) at', name) or re.findall('method (.+) of', name) or [name]
        return name[0]
    
    def 生成md5(text, short=False):
        md5 = hashlib.md5(text.encode(encoding='utf-8')).hexdigest()
        return md5[8:-8] if short else md5

    def 当前时间(): return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    
    def 当前日期(): return time.strftime("%Y-%m-%d", time.localtime())

    def 提取文件目录(path, 后缀=[]):
        files = [str(x) for x in Path(path).iterdir()]
        if 后缀:
            files = [x for x in files if ('.' in x) and (x.split('.')[-1] in 后缀)]
        return files
    
    def 判断中文(text, model='含中文'):
        if type(text) is not str:  # 不是字符串类型, 返回 False
            return False
        elif not text:  # 空字符串, 返回 False
            return False
        elif model == '含中文':  # 只要含有中文, 就返回 True
            for x in text:
                if u'\u4e00' <= x <= u'\u9fa5':
                    return True
            return False
        elif model == '全中文':  # 只要含有非中文, 就返回 False
            for x in text:
                if not (u'\u4e00' <= x <= u'\u9fa5'):
                    return False
            return True
    
    def 循环取值(data):
        while True:
            for x in data:
                yield x

    def 字典取值(item):
        def g(item, i, total, keys):
            if (i < total) and (type(item) is dict):
                ki = keys[i]
                for k, v in item.items():
                    if k == ki:
                        if i == total - 1: return v, True
                        else:
                            value, r = g(v, i + 1, total, keys)
                            if r: return value, True
                    value, r = g(v, i, total, keys)
                    if r: return value, True
            return None, False
        def briskget(keys, 代替值=Exception("briskget未找到key")):
            keys = re.split(' *> *', keys)
            value, r = g(item, 0, len(keys), keys)
            if r: return value
            if type(代替值) is Exception: raise 代替值
            return 代替值
        return briskget

    @classmethod
    def 配置图转字典(cls, 配置图):
        # 先按行分割, 然后每行按#分割并取第1部分, 再删除末尾的空格
        data = [re.sub(" *$", '', x.split('#')[0]) for x in 配置图.split('\n')]
        # 去除无效元素, 然后 计算单项级别 和 提取出单项值
        data = [(len(re.findall('^ *', x)[0])//4+1, re.findall('[^ ].*', x)[0]) for x in data if x.replace(' ', '')]
        # 转为字典
        data = [(a, 'value' if ':' in b else 'list', b) for a, b in data]
        item = {0:{}}
        for n, type_, text in data:
            if type_ == 'list':
                item[n-1][text] = item[n] = {}
            elif type_ == 'value':
                key, value = re.split(' *: *', text, 1)  # 冒号前后自动忽略空格
                item[n-1][key] = eval(value)
        return item[0], cls.字典取值(item[0])



import re
from .dfa import dfa


class phk_dfa(dfa):
    
    网址正则 = '\[[^])([]+\][(]https?://[\w./:]+[)]|https?://[\w./:]+'
    
    def 替换格式(self, 敏感词): return f'[{敏感词}]'

    def 替换(self, text, 敏感词):
        新词 = self.替换格式(敏感词)
        if 新词 in text:  # 替换过了, 不重复替换
            return text
        elif 敏感词 in str(re.findall(self.网址正则, text)):  # 敏感词在网址里面, 不替换
            return text
        return text.replace(敏感词, 新词)
    
    def 敏感词替换(self, text):
        敏感词s = set(self.检测敏感词(text=text, 匹配模式='最长'))
        for 敏感词 in 敏感词s:
            text = self.替换(text, 敏感词)
        return text



# -!- coding: utf-8 -!-
import requests, re, json, time
# 同文件夹模块
from .req_base import req_base
from .req_baidu_pc import req_baidu_pc


self_req_base = req_base()  # 私有变量
class seo_baidu_pc(req_baidu_pc):

    去除www匹配排名 = True
    网站内页排名有效 = True
    psino = False
    psino有效值 = (6, 1, 2)
    无排名报错 = False  # 查询没有排名时报错
    无排名返回 = 'rn+1'  # 无排名时的返回值
    打印查询结果 = True

    results最低占比 = False
    if type(results最低占比) is int: results最低占比 = float(results最低占比)
    # 接收False或浮点数, 范围为0-1, len(results) < rn 时报错, False表示关闭此功能

    url_get_title = self_req_base.url_get_title


    def __init__(self):
        req_baidu_pc.__init__(self)

    def get(self, *vs, **kvs):
        r = req_baidu_pc.get(self, *vs, **kvs)
        if self.psino:
            psino = int(r.cookies.get('PSINO') or -1)
            if psino in self.psino有效值:
                if self.打印查询结果: print(f"psino: {psino}")
            else:
                if self.打印查询结果: print(f"psino: {psino}, 不符合要求")
                raise Exception("psino值不符合要求")
        return r

    # json查排名
    def kw_get_json排名(self, kw, domain, rn=None):
        rn = rn or self.rn
        r = self.json搜索(kw=kw, rn=rn)
        ranking = self.rjson_get_ranking(rjson=r, domain=domain)
        if not ranking:
            if self.无排名报错: raise Exception("self.无排名报错")
            elif self.无排名返回 == 'rn+1': return rn + 1
            else: return self.无排名返回
        if self.打印查询结果: print(f'{kw} - {domain} - {ranking}')
        return ranking

    # 传入 关键词 和 域名 和 熊掌号, 获取pc排名
    def kw_get_ranking(self, kw, domain=None, 熊掌号=None, rn=None, **kvs):
        rn = rn or self.rn
        if domain and domain.count('.') >= 3:
            # 分发到json
            r = self.json搜索(kw=kw, rn=rn)
            ranking = self.rjson_get_ranking(rjson=r, domain=domain)
        else:
            # 分发到普通html
            r = self.搜索(kw=kw, rn=rn)
            results = self.html_get_results(html=self.r_get_html(r=r))
            if type(self.results最低占比) is float:
                if len(results) < rn * self.results最低占比:
                    raise Exception("self.results最低占比")
            ranking = self.results_get_ranking(results=results, domain=domain, 熊掌号=熊掌号)
        if not ranking:
            if self.无排名报错: raise Exception("self.无排名报错")
            elif self.无排名返回 == 'rn+1': return rn + 1
            else: return self.无排名返回
        if self.打印查询结果: print(f'{kw} - {domain} - {熊掌号} - {ranking}')
        return ranking

    # 传入json类型的r和domain, 传出排名
    def rjson_get_ranking(self, rjson, domain):
        domain = self.url_get_domain(url=domain)
        if self.去除www匹配排名: domain = re.sub('^www\.', '', domain)
        rjson = rjson.json()["feed"]["entry"][:-1]
        result = [(x['pn'], re.sub(' |https:|http:|\.shtml|\.html', '', x['url'], 0)) for x in rjson]
        result = [(x[0], re.sub('/+', '/', '/' + x[1] + '/', 0)[1:-1]) for x in result]
        domain = re.sub('/+', '/', '/' + re.sub(' |https:|http:', '', domain, 0) + '/', 0)[1:-1]
        for i in result:
            if domain in i[1]: return int(i[0])
        return None


    #  传入 以排名为key的字典 和 domain, 传出排名
    def results_get_ranking(self, results, domain=None, 熊掌号=None):
        if domain:
            domain = self.url_get_domain(url=domain)
            if self.去除www匹配排名: domain = re.sub('^www\.', '', domain)
        for id, item in results.items():
            底部文字 = item.get('底部文字') or ''
            if domain and domain in self.url_get_domain(url=底部文字):
                if self.网站内页排名有效: return id
                elif not re.findall('/(.+)', re.sub('https?://| ', '', 底部文字)):
                    return id  # 不存在内页, 说明是首页
            if 熊掌号 and 熊掌号 in 底部文字: return id
        return None



# -!- coding: utf-8 -!-
import re, os, sys, requests, json, math, time, copy, random
import pandas as pd
from copy import deepcopy
from collections import Counter
from .shanjianwang_api2_taocan import 善检网api第2版_套餐


class 善检网api第2版_套餐_腾讯(善检网api第2版_套餐):
    
    def __init__(self):
        善检网api第2版_套餐.__init__(self)
        print('腾讯套餐接口')
        self.生成单项字典()
        self.生成机构字典()
        self.机构和统一对应表 = {}
        self.获取标签列表()
        print('实例化完成')

    中文字典 = dict(
        tag_id='标签id', group_name='组名称', tag_code='标签代号', tag_name='标签名称', group_id='组id',
        district_name='区', province_name='省份', org_picture='机构图片', org_address='机构地址',
        org_id='机构id', org_item_id='机构单项id', org_item_name='机构单项名称', org_item_desc='机构单项描述',
        mutual_item_id='统一单项id', mutual_item_name='统一单项名称', mutual_item_desc='统一单项描述',
        city_name='城市', org_level='机构等级', org_rank='机构排名', org_status='机构状态', org_name='机构名称',
        basic = '基础', parent_id = '父级id', description = '描述', genders = '性别',
        package_id = '套餐id', package_name = '套餐名称', package_price = '套餐价格', package_status = '套餐状态',
        big_picture = '大图', small_picture = '小图', sales_volume = '销售量', tag_item = '标签项目',
        create_time = '创建时间', update_time = '更新时间', tag_gender = '标签性别', tag_group = '标签组',
        org_phone = '机构电话', report_types='报告类型',
    )

    ''' 已测试通过 '''
    def 修改基础项(self, 单项id, 单项库类型='统一库', 设为基础项=False):
        ''' 单项库类型: '统一库', '机构库' '''
        updItem2BasicDTO = dict(itemId=单项id, basic=1 if 设为基础项 else 0, type=dict(统一库=1, 机构库=2)[单项库类型])
        r = self.put("tencent/basic-items", data=updItem2BasicDTO)
        msg = r.get('message') or str(r)
        if msg in ['OK', 'SUCCESS', '成功']:
            return '成功', msg
        else:
            return '失败', msg
    
    ''' 已测试通过 '''
    def 查询单项类型(self, 单项级别, 父级id=None, 单项名称=None):
        ''' 单项级别: 1, 2 '''
        params = dict(groupLevel=单项级别, groupName=单项名称, parentId=父级id, per=1)
        params['per'] = self.get("tencent/item-groups", params=params)['total']
        data = self.get("tencent/item-groups", params=params)['data']
        for x in data:
            x['单项id'] = x.pop('组id')
            x['单项名称'] = x.pop('组名称')
        self.转中文(data)
        return data

    ''' 已测试通过 '''
    def 生成单项字典(self):
        print('正在生成单项字典..')
        self.统一单项字典 = item = {x['统一单项id']:x['统一单项名称'] for x in self.查询统一单项()}
        item.update({v:k for k,v in item.items()})
        
        item = {}
        self.单项字典 = 单项字典 = {}
        for x in self.查询单项类型(单项级别=2):
            一级单项id, 二级单项id, 单项名称 = [x[y] for y in ('父级id', '单项id', '单项名称')]
            if 一级单项id not in item:
                item[一级单项id] = {}
            item[一级单项id][二级单项id] = item[一级单项id][单项名称] = dict(二级单项id=二级单项id, 二级单项名称=单项名称)
        for x in self.查询单项类型(单项级别=1):
            一级单项id, 单项名称 = [x[y] for y in ('单项id', '单项名称')]
            单项字典[一级单项id] = 单项字典[单项名称] = dict(
                一级单项id=一级单项id,
                一级单项名称=单项名称,
                二级单项=item.get(一级单项id) or {}
            )
        self.二级单项字典 = {y:dict(一级单项名称=k) for k, x in 单项字典.items() for y in x['二级单项']}

    ''' 已测试通过 '''
    def 新增或修改二级单项(self, 一级单项id, 二级单项id=None, 单项名称=None):
        itemGroupDTO = dict(parentId=一级单项id, groupId=二级单项id, groupName=单项名称)
        r = self.post("tencent/item-group", data=itemGroupDTO)
        if not 二级单项id:  # 如果是新增
            self.生成单项字典()
        msg = r.get('message') or str(r)
        if msg in ['OK', '提交的类型名称已存在,请修改后重新提交', 'SUCCESS', '单项类型名称已存在', '成功']:
            return '成功', msg
        else:
            return '失败', msg

    ''' 已测试通过 '''
    def 查询统一单项(self, 二级单项id=None):
        params = dict(groupId=二级单项id, per=1)
        params['per'] = self.get("tencent/mutual-items", params=params)['total']
        return self.get("tencent/mutual-items", params=params)['data']
    
    ''' 已测试通过 '''
    def 查询统一基础项(self):
        return self.get("tencent/mutual-basic-items")['data']
    
    ''' 已测试通过 '''
    def 新增或修改统一单项(self, 二级单项id, 单项名称, 单项id=None, 描述=None, 设为基础项=None):
        mutualItemDTO = dict(groupId=二级单项id, mutualItemId=单项id, mutualItemName=单项名称)
        if 描述 != None:
            mutualItemDTO['mutualItemDesc'] = 描述
        if 设为基础项 != None:
            mutualItemDTO['basic'] = 1 if 设为基础项 else 0
        r = self.post("tencent/mutual-item", data=mutualItemDTO)
        msg = r.get('message') or str(r)
        if msg in ['OK', '提交的类型名称已存在,请修改后重新提交', '成功', '统一单项名称已存在', 'SUCCESS']:
            return '成功', msg
        else:
            return '失败', msg
    
    ''' 已测试通过 '''
    def 批量新增统一单项(self, kws, 一级字段='一级', 二级字段='二级', 三级字段='三级', 设为基础项=False):
        type_kws = type(kws)
        if type_kws is pd.core.frame.DataFrame:
            kws = json.loads(kws.to_json(orient='records'))
        elif type_kws is str:
            df = pd.read_excel(kws)
            重复的 = [k for k, v in dict(Counter([a[1] for a, b in df.groupby([一级字段, 二级字段])])).items() if v > 1]
            if 重复的:
                print(重复的)
                return None
            kws = json.loads(df.to_json(orient='records'))
        for x in kws:
            一级 = self.二级单项字典.get(x['二级'], {}).get('一级单项名称') or None
            if x['一级'] not in self.单项字典:
                print(f"一级单项 {x['一级']} 不存在")
                强制报错
            if 一级 and 一级 != x['一级']:
                print(f"二级单项 {x['二级']} 已存在于 {一级}中, 无法向 {x['一级']} 添加")
                强制报错
        kws = [x for x in kws if x['三级'] not in self.统一单项字典]
        total = len(kws)
        for count, x in enumerate(kws):
            一级单项名称, 二级单项名称, 三级单项名称 = [x[y] for y in (一级字段, 二级字段, 三级字段)]
            二级单项 = self.单项字典[一级单项名称]['二级单项'].get(二级单项名称)
            if 二级单项:
                二级单项id = 二级单项['二级单项id']
            else:
                print(f'\r{x} - 二级单项不存在, 将自动创建二级单项', end='')
                一级单项id = self.单项字典[一级单项名称]['一级单项id']
                self.新增或修改二级单项(一级单项id=一级单项id, 单项名称=二级单项名称)
                二级单项id = self.单项字典[一级单项名称]['二级单项'][二级单项名称]['二级单项id']
            x['状态'], x['msg'] = self.新增或修改统一单项(二级单项id=二级单项id, 单项名称=三级单项名称, 设为基础项=设为基础项)
        print()
        self.生成单项字典()
        return kws
    
    ''' 已测试通过 '''
    def 查询机构基础项(self, 机构id):
        params = dict(orgId=机构id)
    
    ''' 已测试通过 '''
    def 新增或修改机构单项(self, 机构id, 二级单项id, 单项名称, 单项id=None, 设为基础项=None, 描述=None):
        orgItemDTO = dict(orgId=机构id, groupId=二级单项id, orgItemName=单项名称, orgItemId=单项id)
        if 描述 != None:
            orgItemDTO['orgItemDesc'] = 描述
        if 设为基础项 != None:
            orgItemDTO['basic'] = 1 if 设为基础项 else 0
        r = self.post("tencent/org-item", data=orgItemDTO)
        msg = r.get('message') or str(r)
        if msg in ['OK', '该机构下已存在当前项目名称，请核对数据', '成功', 'SUCCESS']:
            return '成功', msg
        else:
            return '失败', msg

    ''' 已测试通过 '''
    def 查询机构单项(self, 二级单项id=None, 机构id=None, 单项名称=None):
        params = dict(groupId=二级单项id, orgId=机构id, orgItemName=单项名称, per=1)
        params['per'] = self.get("tencent/org-items", params=params)['total']
        return self.get("tencent/org-items", params=params)['data']

    
    ''' 已测试通过 '''
    def 套餐列表(self, 机构id, 套餐名称=None, 上架=None, 创建时间范围=(None, None)):
        params = dict(orgId=机构id, bizField=套餐名称, per=1)
        if 上架 != None:
            params['status'] = 1 if 上架 else 0
        params['per'] = self.get("tencent/packages", params=params)['total']
        data = self.get("tencent/packages", params=params)['data']
        f = lambda x: time.mktime(time.strptime(x, "%Y-%m-%d %H:%M:%S"))
        start_time = f(创建时间范围[0] or '1999-10-10 10:10:10')
        end_time = f(创建时间范围[1] or '2999-10-10 10:10:10')
        return [x for x in data if start_time <= f(x['创建时间']) <= end_time]
    
    ''' 已测试通过 '''
    def 套餐详情(self, 套餐id):
        params = dict(packageId=套餐id)
        data = self.get("tencent/package", params=params)['data']
        data['items'] = {x['label']: x['data']for x in data['items']}
        return data
    
    ''' 已测试通过 '''
    def 获取机构列表(self, 状态=None):
        params = dict()
        if 状态 != None:
            params['status'] = 1 if 状态 else 0
        params['per'] = self.get("tencent/orgs", params=params)['total']
        return self.get("tencent/orgs", params=params)['data']
    
    ''' 已测试通过 '''
    def 获取机构详情(self, 机构id):
        params = dict(orgId=机构id)
        data = self.get("tencent/org", params=params)['data']
        item = [('org_province', '省份id'), ('org_city', '城市id'), ('org_district', '区id')]
        for a, b in item:
            data[b] = data.pop(a) if a in data else None
        return data
    
    ''' 已测试通过 '''
    def 生成机构单项和统一单项对应表(self, 机构id):
        item = {}
        for 一级单项 in self.get("tencent/package-items", params=dict(orgId=机构id))['data']:
#             item[一级单项['组名称']] = item[一级单项['组id']] = x_item = {}
            for 二级单项 in 一级单项['group_lv2']:
#                 x_item[二级单项['组名称']] = x_item[二级单项['组id']] = y_item = {}
                for x in 二级单项['items']:
#                     机构单项id, 机构单项名称 = [f"机构单项_{三级单项[z]}" for z in ('机构单项id', '机构单项名称')]
#                     统一单项id, 统一单项名称 = [f"统一单项_{三级单项[z]}" for z in ('统一单项id', '统一单项名称')]
#                     y_item[机构单项id] = y_item[机构单项名称] = y_item[统一单项id] = y_item[统一单项名称] = 三级单项
                    item[f"机构单项id_{x['机构单项id']}"] = x
                    item[f"统一单项id_{x['统一单项id']}"] = x
                    item[f"机构单项名称_{x['机构单项名称']}"] = x
                    item[f"统一单项名称_{x['统一单项名称']}"] = x
        self.机构和统一对应表[机构id] = item
        return item
    
    ''' 已测试通过 '''
    def 拉取统一单项到机构单项(self, 机构id, 统一单项ids):
        统一单项ids = [dict(mutualItemId=x) for x in 统一单项ids]
        data = dict(orgId=机构id, items=统一单项ids)
        r = self.post("tencent/push-org-item", data=data)
        self.生成机构单项和统一单项对应表(机构id=机构id)
        msg = r.get('message') or str(r)
        if msg in ['OK', '成功', 'SUCCESS']:
            return '成功', msg
        else:
            return '失败', msg

    ''' 已测试通过 '''
    def 生成机构字典(self):
        print('正在生成机构字典..')
        self.机构字典 = 机构字典 = {}
        for x in self.获取机构列表():
            机构id, 机构名称, 机构状态 = [x[y] for y in ('机构id', '机构名称', '机构状态')]
            机构字典[机构id] = 机构字典[机构名称] = dict(机构id=机构id, 机构名称=机构名称, 机构状态=机构状态)
    
    ''' 已测试通过 '''
    def 删除套餐(self, 机构id, 套餐id):
        data = {'bigPicture': None, 'genders': None, 'orgId': 机构id, 'packageName': 'test', 'packageId': 套餐id,
                'packagePrice': 99999999.0, 'packageStatus': 0, 'packagesItems': None, 'smallPicture': None}
        r = self.put("tencent/package", data=data)
        msg = r.get('message') or str(r)[:100]
        if msg in ['OK', 'SUCCESS', '成功']:
            return '成功', msg
        else:
            return '失败', msg

    def 匹配特色标签(self, 项目):
        项目规则 = [
            ('外科', '外科'), ('肝功', '肝功能'), ('糖尿', '糖尿病'), ('眼', '眼科'), ('基因', '基因检测'),
            ('肿瘤和癌', '肿瘤筛查'), ('胃镜', '胃镜'), ('肠镜', '肠镜'), ('心肌', '心肌功能'),
            ('肾功', '肾功能'), ('甲功', '甲状腺功能')
        ]
        return [y for x, y in 项目规则 if x in 项目]
    
    ''' 已测试通过 '''
    def 清空机构套餐(self, 机构id):
        if type(机构id) is str:
            机构id = self.机构字典[机构id]['机构id']
        rs = []
        for x in [x['套餐id'] for x in self.套餐列表(机构id=机构id)]:
            rs.append(self.删除套餐(机构id=机构id, 套餐id=x))
            print(set(rs))
    
    ''' 已测试通过 '''
    def 新增套餐(self, 机构s, 套餐名称字段, 性别字段, 套餐价格字段, 年龄字段,
                图片字段, 单项字段, 上架字段, 套餐id字段=None, 描述字段=None):
        套餐性别字典 = {x['标签名称']:x['标签代号'] for x in self.获取标签列表('package_gender')}
        套餐性别字典['已婚女'] = 套餐性别字典['女已婚']
        套餐性别字典['未婚女'] = 套餐性别字典['女未婚']
        alldata = []
        for 机构名称, 套餐s in 机构s.items():
            机构id = self.机构字典[机构名称]['机构id']
            jt对应表 = self.生成机构单项和统一单项对应表(机构id)
            
            for 套餐 in 套餐s:
                套餐id, 套餐名称, 描述 = [套餐.get(x) for x in (套餐id字段, 套餐名称字段, 描述字段)]
                
                性别 = [套餐性别字典[x] for x in set(套餐.get(性别字段) or [])]
                性别中文 = [x for x in set(套餐.get(性别字段) or [])]
                套餐价格 = 套餐[套餐价格字段] * 100
                上架 = 1 if 套餐.get(上架字段) else 0
                
                # 年龄标签
                年龄 = list(套餐.get(年龄字段) or [])
                if ('入职' in 套餐名称) or ('招工' in 套餐名称):
                    年龄.append('入职体检')
                if ('婚前' in 套餐名称) or ('孕前' in 套餐名称) or ('优生' in 套餐名称):
                    年龄.append('婚检孕检')
                年龄 = set(年龄)
                

                图片 = random.sample(self.图片库[套餐.get(图片字段)], 1)[0]
                大图 = f"https://img.tijian8.com/tha/package/origin/{图片}"
                小图 = f"https://img.tijian8.com/tha/package/sm/{图片.replace('.', '_SM.')}"
                
                单项参数 = []
                单项s = set(套餐.get(单项字段) or [])
                for 单项 in 单项s:
                    统一单项id = self.统一单项字典.get(单项) or self.统一单项字典.get(re.sub(' +$', '', 单项)) or self.统一单项字典[单项]
                    机构单项id = jt对应表.get(f"统一单项id_{统一单项id}", {}).get('机构单项id')
                    if not 机构单项id:
                        self.拉取统一单项到机构单项(机构id=机构id, 统一单项ids=[统一单项id])
                        机构单项id = self.机构和统一对应表[机构id][f"统一单项id_{统一单项id}"]['机构单项id']
                    单项参数.append(dict(mutualItemId=统一单项id, orgItemId=机构单项id))
                项目标签 = self.匹配特色标签(项目=''.join(单项s).replace(' ', ''))

                self.重置套餐标签(套餐id=None, 项目标签=项目标签, 人群标签=年龄, 性别标签=性别, 上传=False)  # 只验证, 不上传

                data = dict(bigPicture=大图, genders=性别, orgId=机构id, packageName=套餐名称, packagePrice=套餐价格,
                            packageStatus=上架, packagesItems=单项参数, smallPicture=小图)
                if 描述 != None:
                    data['description'] = 描述
                if 套餐id != None:
                    data['packageId'] = 套餐id
                
                alldata.append((data, 年龄, 项目标签, 性别))

        成功, 上传套餐失败, 添加标签失败 = [], [], []
        for data, 年龄, 项目标签, 性别 in alldata:
            机构id, 套餐名称 = data['orgId'], data['packageName']
            套餐id_old = set([x['套餐id'] for x in self.套餐列表(机构id=机构id, 套餐名称=套餐名称)])
            
            r = self.put("tencent/package", data=data)
            msg = r.get('message') or str(r)[:100]
            print(机构id, 套餐名称, msg)
            
            if msg in ['OK', 'SUCCESS', '成功']:
                套餐id_new = set([x['套餐id'] for x in self.套餐列表(机构id=机构id, 套餐名称=套餐名称)])
                套餐id = list(套餐id_new - 套餐id_old)
                if len(套餐id) == 1:
                    套餐id = 套餐id[0]
                    标签状态, 标签msg = self.重置套餐标签(套餐id=套餐id, 项目标签=项目标签, 人群标签=年龄, 性别标签=性别, 上传=True)
                    if 标签状态 == '成功':
                        成功.append((msg, data, 年龄, 项目标签, 性别, 标签msg))
                    else:
                        添加标签失败.append((msg, data, 年龄, 项目标签, 性别, 标签msg))
                else:
                    添加标签失败.append((msg, data, 年龄, 项目标签, 性别, f'套餐id:{套餐id}'))
            else:
                上传套餐失败.append((msg, data, 年龄, 项目标签, 性别))
        return dict(成功=成功, 上传套餐失败=上传套餐失败, 添加标签失败=添加标签失败)
    
    ''' 已测试通过 '''
    def 获取标签列表(self, group_key=None):
        r = self.get("tencent/tag/List", params=dict(groupKey=None))['data']
        self.标签字典 = item = {}
        for x in r:
            type_ = x['group_key']
            if type_ not in item:
                item[type_] = {}
            item[type_][x['标签id']] = item[type_][x['标签名称']] = item[type_][x['标签代号']] = x
        item['项目'] = item['package_item']
        item['人群'] = item['package_group']
        item['性别'] = item['package_gender']
        item['机构等级'] = item['org_rank']
        item['机构类型'] = item['org_level']
        if group_key:
            return [x for x in r if x['group_key'] == group_key]
        else:
            return r

    def 新增标签(self, 标签组, 标签代号, 标签名称):
        data = dict(groupKey=标签组, tagCode=标签代号, tagName=标签名称)
        r = self.post("tencent/tag", data=data)
        msg = r.get('message') or str(r)
        if msg in ['OK', 'SUCCESS', '成功']:
            self.获取标签列表()
            return '成功', msg
        else:
            return '失败', msg
    
    ''' 已测试通过 '''
    def 从Excel上传套餐(self, path, 唯一识别字段='唯一识别', 机构字段='医院名称', 套餐名称字段='套餐名称',
                   单项名称字段='单项库叫法', 上架字段='上架', 套餐价格字段='价格', 图片字段='图片', 性别字段='性别',
                  年龄字段='适用年龄', 清空旧套餐=False, 挂网折扣字段='挂网折扣'):
        df = pd.read_excel(path)
        
        # 检查字段是否齐全
        字段s = df.columns.to_list()
        for x in ('唯一识别', '医院名称', '套餐名称', '单项库叫法', '价格', '图片', '性别', '适用年龄', '挂网折扣'):
            if x not in 字段s:
                print(f"字段 [{x}] 不在Excel文档中")
                强制报错
        
        # 判断机构是否重复
        机构s = set(df[机构字段].to_list())
        if len(机构s) > 1:
            print(f"机构名称不一致: {机构s}")
            强制报错
        else:
            机构名称 = list(机构s)[0]
        
        # 提取所需数据
        机构s = {}
        for 标识符, df in df.groupby(唯一识别字段):
            for 机构名称, df in df.groupby(机构字段):
                if 机构名称 in 机构s:
                    套餐s = 机构s[机构名称]
                else:
                    机构s[机构名称] = 套餐s = []
                for 套餐名称, df in df.groupby(套餐名称字段):
                    套餐 = df.iloc[0].to_dict()
                    套餐[单项名称字段] = re.split(' *\[SEP\] *| *\[SEG\] *', '[SEP]'.join(list(df[单项名称字段])).replace('|', '[SEP]'))
                    套餐[上架字段] = True if 套餐.get(上架字段, '是') in ['是', True, 1] else False
                    套餐[性别字段] = [y for x in list(df[性别字段]) for y in re.split(' +', x)]
                    套餐[年龄字段] = [y for x in list(df[年龄字段]) for y in re.split(' +', x)]
                    套餐[套餐价格字段] = float(套餐[套餐价格字段]) * float(套餐[挂网折扣字段]) / 100
                    套餐s.append(套餐)
        
        if 清空旧套餐:
            for 机构名称 in 机构s:
                self.清空机构套餐(机构id=机构名称)
        
        return self.新增套餐(机构s=机构s, 套餐名称字段=套餐名称字段, 性别字段=性别字段, 套餐价格字段=套餐价格字段,
                图片字段=图片字段, 单项字段=单项名称字段, 上架字段=上架字段, 年龄字段=年龄字段)
    
    ''' 已测试通过 '''
    def 重置套餐标签(self, 套餐id, 项目标签=[], 人群标签=[], 性别标签=[], 上传=True):  # 同类型的原有标签将被清空
        项目标签代号 = [self.标签字典['项目'][x]['标签代号'] for x in set(项目标签 or [])]
        人群标签代号 = [self.标签字典['人群'][x]['标签代号'] for x in set(人群标签 or [])]
        性别标签代号 = [self.标签字典['性别'][x]['标签代号'] for x in set(性别标签 or [])]
        if 上传:
            rs = {}
            if 项目标签代号:
                data = dict(codes=项目标签代号, groupKey=2, packageId=套餐id)
                rs['项目'] = self.post("tencent/package/tag", data=data)
            if 性别标签代号:
                data = dict(codes=性别标签代号, groupKey=0, packageId=套餐id)
                rs['性别'] = self.post("tencent/package/tag", data=data)
            if 人群标签代号:
                data = dict(codes=人群标签代号, groupKey=1, packageId=套餐id)
                rs['人群'] = self.post("tencent/package/tag", data=data)
            for r in rs.values():
                if r['message'] not in ['SUCCESS', '成功']:
                    return '失败', str(rs)
            else:
                return '成功', str(rs)
    
    def 套餐转参数(self, 机构id, 套餐id):
        t = self.套餐详情(套餐id=套餐id)
        data = dict(
            bigPicture = t['大图'],
            description = t['描述'],
            genders = [x['gender_code'] for x in t['性别']],
            orgId = 机构id,
            packageId = 套餐id,
            packageName = t['套餐名称'],
            packagePrice = t['套餐价格'],
            packageStatus = t['套餐状态'],
            packagesItems = [dict(mutualItemId=y['统一单项id'], orgItemId=y['机构单项id']) for x in t['items'].values() for y in x],
            salesVolume = t['销售量'],
            smallPicture = t['小图']
        )
        
        bq = t['tags']
        标签 = dict(
            套餐id = 套餐id,
            项目标签 = [x['text'] for x in bq['标签项目']],
            人群标签 = [x['text'] for x in bq['标签组']],
            性别标签 = [x['text'] for x in bq['标签性别']]
        )
        return dict(套餐=data, 标签=标签)

    def 修改机构信息(self, 机构id, olddata=None, **kvs):
        if True:
            未验证
        kvs = {k: v for k, v in kvs.items() if v is not None}
        data = olddata or self.获取机构详情(机构id=机构id)
        newdata = dict(
            cityName = kvs.get('城市名称', data['城市']),
            districtName = kvs.get('地区名称', data['区']),
            orgAddress = kvs.get('地址', data['机构地址']),
            orgCity = kvs.get('城市id', data['城市id']),
            orgDistrict = kvs.get('地区id', data['区id']),
            orgId = 机构id,
            orgLevel = kvs.get('机构等级', data['机构等级']),
            orgName = kvs.get('机构名称', data['机构名称']),
            orgPhone = kvs.get('机构电话', data['机构电话']),
            orgPicture = kvs.get('机构图片', data['机构图片']),
            orgProvince = kvs.get('省份id', data['省份id']),
            orgRank = kvs.get('机构排名', data['机构排名']),
            orgStatus = kvs.get('机构状态', data['机构状态']),
            provinceName = kvs.get('省份', data['省份']),
            reportTypes = ','.join([x['id'] for x in data['报告类型']]),
        )
        r = self.put("alibaba/package", data=newdata)
        if r['message'] in ('SUCCESS', ):
            return True
        else:
            print(机构id, r)
            return dict(机构id=机构id, 参数=deepcopy(newdata), r=r)



import re, os, sys, time
from apscheduler.schedulers.blocking import BlockingScheduler

class 自动任务():
    最大实例数量 = 100
    
    def __init__(self): self.任务池 = BlockingScheduler()
    
    # 间隔, 并非从任务完成后开始计算, 而是从任务开始时开始计算
    # 当 'max_instances' 键值对的值小于 任务时长/任务间隔 时，程序会报错。
    def 添加间隔任务(self, func, 间隔=(None, ) * 4, 开始时间=None, 结束时间=None):
        item = dict(func=func, trigger='interval', seconds=间隔, max_instances=self.最大实例数量)
        for k, v in zip(('days', 'hours', 'minutes', 'seconds'), 间隔):
            if v: item[k] = v
        if 结束时间: item['end_date'] = 结束时间  # '2019-11-30 21:30:00'
        十秒前 = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time() - 10))
        item['start_date'] = 开始时间 or 十秒前  # '2019-10-30 21:30:00'
        self.任务池.add_job(**item)
    
    def 添加周期任务(self, func, 周期=(None, ) * 6, 开始时间=None, 结束时间=None):
        item = dict(func=func, trigger='cron', max_instances=self.最大实例数量)
        for k, v in zip(('year', 'month', 'day', 'hour', 'minute', 'second'), 周期):
            if v: item[k] = v  # month='3-4,7-9', day='1-3', hour='3'
        if 开始时间: item['start_date'] = 开始时间  # '2019-11-30 21:30:00'
        if 结束时间: item['end_date'] = 结束时间
        self.任务池.add_job(**item)
    
    def 添加一次性任务(self, func, 时间):
        # 时间: '2019-10-30 21:30:00'
        self.任务池.add_job(func=func, trigger='date', run_date=时间, max_instances=self.最大实例数量)
    
    def 开始(self): self.任务池.start()



# -!- coding: utf-8 -!-
import time, json, pymysql, traceback
import pandas as pd
import numpy as np
from math import ceil

class pmysql():
    mysql账号 = {}
    持久连接 = False
    数据分组尺寸 = 1000
    
    def __init__(self, mysql账号={}):  # {代号: {账号, 数据库, 数据表}}
        if mysql账号: self.mysql账号 = mysql账号
        self.快速读写地址 = self.mysql账号.get('快速读写', {})
    
    def 读(self, id): return self.快速读写(id=id, model='读')
    def 写(self, id, content): return self.快速读写(id=id, model='写', content=content)
    def 快速读写(self, id, model, content=None):
        nowtime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        for n in range(3):
            try:
                conn = pymysql.connect(**self.快速读写地址['账号'], db=self.快速读写地址['数据库'])
                cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
                if model == '读':
                    sql = f"select content from {self.快速读写地址['数据表']} where id = {id}"
                elif model == '写':
                    content = json.dumps(content, ensure_ascii=False)
                    sql = f"update {self.快速读写地址['数据表']} set content='{content}', 更新时间='{nowtime}' where id = {id}"
                cursor.execute(sql)
                data = cursor.fetchall()
                data = json.loads(data[0]['content'] or "null") if data else None
                # data[0]['content'] 一定是字符串, 空字符串也即null
                conn.commit()
                if not self.持久连接: conn.close()
                return data
            except:
                try:
                    if not self.持久连接: conn.close()
                except: pass
                time.sleep(1)
    
    def 创建连接(self, 地址):
        for i in range(5):
            try:
                conn = pymysql.connect(**地址['账号'], db=地址['数据库'])
                cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
                return conn, cursor
            except Exception as err:
                e = err
                time.sleep(1)
        raise e
    
    def 操作数据表(self, 代号, sql=None, **kw):
        数据类型 = kw.get('数据类型', 'list').lower()
        地址 = self.mysql账号[代号] if type(代号) is str else 代号
        conn, cursor = self.创建连接(地址=地址)
        if sql:
            sql = sql.replace('{数据表}', 地址['数据表'])
            cursor.execute(sql)
            datas = list(cursor.fetchall())
            conn.commit()
            if not self.持久连接: conn.close()
            if 数据类型 in ('pandas', 'df'): return pd.DataFrame(datas)
            return datas
        return conn, cursor, 地址['数据表']
    
    def get字段(self, 代号):
        sql字段 = self.操作数据表(代号=代号, sql="select column_name as name from information_schema.columns where table_name = '{数据表}'")
        return [x['name'] for x in sql字段]
    
    def 数据清洗(self, 代号, datas, **kw):
        kvs = kw.get('kvs') or {}
        忽略字段 = kw.get('忽略字段') or set()
        df = datas if (type(datas) is pd.core.frame.DataFrame) else pd.DataFrame(datas)
        for k, v in kvs.items(): df[k] = v
        字段s = set(df.columns) - set(忽略字段) & set(self.get字段(代号=代号))
        df = pd.DataFrame(df, columns=字段s)
        datas = json.loads(df.to_json(orient='records'))
        return dict(datas=datas, 字段s=字段s)
    
    def 添加数据(self, 代号, datas, 极致=True, 忽略字段=set(), **kw):
        调试 = kw.get('调试', True)
        size = kw.get('size') or self.数据分组尺寸
        kvs = kw.get('kvs')
        打印状态 = kw.get('打印状态', True)
        fails = []
        if (type(datas) is pd.core.frame.DataFrame) or datas:
            datas = self.数据清洗(代号=代号, datas=datas, 忽略字段=忽略字段, kvs=kvs)
            datas, 字段s = [datas[x] for x in ('datas', '字段s')]
            # 初始化数据库连接
            conn, cursor, 数据表 = self.操作数据表(代号=代号) 
            sql = f"insert into {数据表}({', '.join(字段s)}) VALUES ({', '.join(('%s',)*len(字段s))})"
            # 批量上传
            datas = [datas[size*(i-1): size*i] for i in range(1, ceil(len(datas)/size)+1)]
            group_number = len(datas)
            for i, idatas in enumerate(datas):
                if 打印状态: print(f'\r正在批量上传第 {i+1}/{group_number} 组, 每组 {size} 条', end='')
                values = tuple([tuple(x.values()) for x in idatas])
                try:
                    cursor.executemany(sql, values)
                    conn.commit()
                except:
                    conn.rollback()
                    fails += idatas
                    if 打印状态: print('本组插入失败, 已回滚')
            # 逐条上传
            if 极致 and fails:
                if 打印状态: print('\n已开启极致模式, 开始逐条上传')
                fails, datas, total = [], fails, len(fails)
                for count, data in enumerate(datas):
                    if 打印状态: print(f'\r正在逐条上传第 {count+1}/{total} 条', end='')
                    try: cursor.execute(sql, tuple(data.values()))
                    except:
                        if 调试: traceback.print_exc()
                        fails.append(data)
                    if (count+1) % size == 0: conn.commit()
                conn.commit()
            # 关闭数据库连接
            try:
                if not self.持久连接: conn.close()
            except: pass
        if 打印状态: print(f'\n\n上传完成, 上传失败共 {len(fails)} 条')
        return fails
    
    def 修改数据(self, 代号, datas, pro_key, **kw):
        调试 = kw.get('调试', False)
        忽略字段 = kw.get('忽略字段', set())
        size = kw.get('size', 1000)
        打印状态 = kw.get('打印状态', True)
        fails = []
        if datas:
            字段s = set(self.get字段(代号=代号)) - set(忽略字段) - {pro_key}
            datas = [(c+1, x[pro_key], {k:v for k,v in x.items() if k in 字段s}) for c,x in enumerate(datas)]
            total = len(datas)
            conn, cursor, 数据表 = self.操作数据表(代号=代号)
            for count, pro_v, data in datas:
                if 打印状态: print(f'\r正在逐条修改第 {count}/{total} 条', end='')
                updates = ','.join([f"{k}={json.dumps(v, ensure_ascii=False)}" for k, v in data.items()])
                pro_v = json.dumps(pro_v, ensure_ascii=False)
                try:
                    sql = f"update {数据表} set {updates} where {pro_key} = {pro_v}"
                    cursor.execute(sql)
                except:
                    fails.append(data)
                    if 调试: traceback.print_exc()
                if count % size == 0: conn.commit()
            conn.commit()
            try:
                if not self.持久连接: conn.close()
            except: pass
        if 打印状态: print(f'\n\n修改完成, 修改失败共 {len(fails)} 条')
        return fails
    
    # def 数据表迁移(self, 原表, 新表, where条件, 迁移字段='*', 去除主键='id', 模式='复制', 字段映射=None):
    #     sql = f"select {迁移字段} from 数据表 where {where条件}" if where条件 else f"select {迁移字段} from 数据表"
    #     sql = sql.replace(' from 数据表', ' from {数据表}')
    #     datas = self.操作数据表(代号=原表, sql=sql)
    #     if datas:
    #         if 去除主键 in datas[0]:
    #             for data in datas:
    #                 data.pop(去除主键)
    #         if 字段映射:
    #             for data in datas:
    #                 for oldkey, newkey in 字段映射.items():
    #                     data[newkey] = data.pop(oldkey)
    #         fails = self.添加数据(代号=新表, datas=datas, 极致=True)
    #         if 模式 in ['剪切', '剪贴', 'cut']:
    #             sql = f"delete from 数据表 where {where条件}" if where条件 else f"delete from 数据表"
    #             sql = sql.replace(' from 数据表', ' from {数据表}')
    #             self.操作数据表(代号=原表, sql=sql)
    #         return fails

    def 创建数据库(self, 代号, 数据库):
        地址 = self.mysql账号[代号] if type(代号) is str else 代号
        for i in range(5):
            try:
                conn = pymysql.connect(**地址['账号'])
                cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
            except:
                time.sleep(1)
        cursor.execute(f"create database {数据库}")
        conn.commit()
        if not self.持久连接: conn.close()
        return True
    
    # def 创建数据表(self, 代号, 数据表, 字段s):
    #     地址 = self.mysql账号[代号] if type(代号) is str else 代号
    #     for i in range(5):
    #         try:
    #             conn = pymysql.connect(**地址['账号'], db=地址['数据库'])
    #             cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
    #         except:
    #             time.sleep(1)
    #     sql = f"create table {数据表} ({字段s}) engine=innodb charset=utf8mb4"
    #     cursor.execute(sql)
    #     conn.commit()
    #     if not self.持久连接: conn.close()
    #     return True
    
    # def 如何创建数据表(self):
    #     print('''
    #     create table 拟定的数据表名称 (
    #     id int(100) primary key auto_increment not null,
    #     添加日期 datetime,
    #     说明 varchar(1000)
    #     ) engine=innodb charset=utf8mb4
    #     ''')



import requests, time, re

class 起点系统爬虫():
    cookie = ''  # 只要账号密码不修改, cookie就不需要更换
    
    def __init__(self, cookie=None):
        if cookie: self.cookie = cookie
    
    def 获取关键词(self, **kw):
        url = 'http://youhua.gsyjiao.com/API/Words/RankDataByDay'
        headers = {
            'ccept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Connection': 'keep-alive',
            'Content-Length': '123',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Cookie': self.cookie,
            'Host': 'youhua.gsyjiao.com',
            'Origin': 'http://youhua.gsyjiao.com',
            'Referer': 'http://youhua.gsyjiao.com/New/WordsStat/List',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36',
            'X-Requested-With': 'XMLHttpReques'
        }
        data  = dict(
            wordName = None, domainName = None, Url = None, comName = None,
            rankDate1 = None,  # 查询日期 '2020-09-16'
            rankNum = 0, undefined = -1, rankType = -1, WordsType = -1,
            page = 1, pagesize = 15,  # 最大不能超过5000
        )
        data.update(kw)
        if data['pagesize'] > 5000:
            print('最大不能超过5000')
            return None
        if not data['rankDate1']:
            print('rankDate1查询日期不能为空, 格式: 2020-09-16')
            return None
        r = requests.post(url, headers=headers, data=data).json()
        for x in r['Items']: x['kwid'] = x.pop('WordID')
        return r

    def 智能提取关键词(self, 日期):
        kws = []
        page_total = ''
        for page in range(1, 100):
            print(f'\r正在获取第{page}/{page_total}页    ', end='')
            ikws = self.获取关键词(rankDate1=日期, page=page, pagesize=5000)
            kws += ikws['Items']
            page_total = ikws['TotalPageCount']
            if page >= page_total:
                break
        print('\n提取完成')
        return kws


class 代理商关键词():
    cookie = ''
    
    def __init__(self, cookie=None):
        if cookie: self.cookie = cookie
    
    def 获取关键词(self, 代理商=None, pagesize=None, page=None):
        url = 'http://youhua.gsyjiao.com/API/Words/ComAgentWordsData'
        headers = {
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Connection': 'keep-alive',
            'Content-Length': '174',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Cookie': self.cookie,
            'Host': 'youhua.gsyjiao.com',
            'Origin': 'http://youhua.gsyjiao.com',
            'Referer': 'http://youhua.gsyjiao.com/New/Words/ComAgentWords',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36',
            'X-Requested-With': 'XMLHttpRequest'
        }
        data = dict(
            wordName = None, domainName = None, Url = None,
            LoginName = None, comName = None, onlyAgent = 0, ComAgent = 1, reachType = None,
            undefined = -1, rankType = -1, auditState = 99, IsEnabled = -1, rankNum = 0, WordsType = -1,
            page = 1,
            pagesize = 15,  # 最大不能超过5000
        )
        if 代理商: data['comName'] = 代理商
        if pagesize: data['pagesize'] = pagesize
        if page: data['page'] = page
        if data['pagesize'] > 5000: raise Exception('pagesize 最大不能超过5000')
        for i in range(20):
            try:
                r = requests.post(url, headers=headers, data=data).json()
                break
            except Exception as err:
                e = err
                time.sleep(20)
        else: raise e
        
        for x in r['Items']:
            x['代理商'] = x.pop('ComName')
            x['kwid'] = x.pop('UID')
            x['kw'] = x.pop('Name')
            x['domain'] = x.pop('DomainName')
            try:
                排名时间 = int(re.findall('\((\d+)\)', x.pop('RankTime'))[0]) / 1000
                x['排名时间'] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(排名时间))
            except:
                x['排名时间'] = ''
        return r

    def 智能提取关键词(self, 代理商=None, count=None):
        kws = []
        page_total = ''
        pagesize = min(count or 1000, 1000)
        for page in range(1, 100000):
            print(f'\r正在获取第{page}{page_total}页', end='    ')
            ikws = self.获取关键词(代理商=代理商, pagesize=pagesize, page=page)
            for x in ikws['Items']: kws.append(x)
            page_total = f"/{ikws['TotalPageCount']}"
            if page >= ikws['TotalPageCount']: break
            if count and (len(kws) >= count): break
        if count and (len(kws) > count):
            kws = kws[:count]
        print('\n提取完成')
        return kws



# -!- coding: utf-8 -!-
import re, os, sys, requests, json, math, time, copy, random
import pandas as pd
import numpy as np
from copy import deepcopy
from collections import Counter
from functools import reduce
from collections import Counter
from random import sample


class 供应链api():
    domain='http://112.74.78.240:31001'
    账号, 密码 = '', ''
    
    def __init__(self, 账号=None, 密码=None):
        if 账号: self.账号 = 账号
        if 密码: self.密码 = 密码
        self.更新token()
        self.获取一级单项列表()
        self.获取二级单项列表()
        self.获取系统单项列表()
        self.生成标签字典()
        # 先获取品牌列表, 再获取机构列表
        self.获取品牌列表()
        self.获取机构列表()
        print('\n实例化完成')
    
    def 更新token(self):
        url = f'{self.domain}/service-scm/login'
        headers = {'Content-Type': 'application/json'}
        data = json.dumps({"account": self.账号, "password": self.密码})
        r = requests.post(url, data=data, headers=headers)
        self.token = r.json()['data']['token']
    
    # 图片库
    图片库 = {
        '中年人': ('ZhongNian1.jpg', 'ZhongNian10.jpg', 'ZhongNian2.jpg', 'ZhongNian3.jpg', 'ZhongNian4.jpg', 'ZhongNian5.jpg', 'ZhongNian6.jpg', 'ZhongNian7.jpg', 'ZhongNian8.jpg', 'ZhongNian9.jpg', 'ZhongNianRen.jpg', 'ZhongNianRen2.jpg', 'ZhongNianRen3.jpg', 'ZhongNianRen4.jpg', 'ZhongNianRen5.jpg'),
        '入职': ('RuZhi.jpg', 'RuZhi10.jpg', 'RuZhi11.jpg', 'RuZhi12.jpg', 'RuZhi13.jpg', 'RuZhi14.jpg', 'RuZhi15.jpg', 'RuZhi2.jpg', 'RuZhi3.jpg', 'RuZhi4.jpg', 'RuZhi5.jpg', 'RuZhi6.jpg', 'RuZhi7.jpg', 'RuZhi8.jpg', 'RuZhi9.jpg'),
        '公务员': ('GongWuYuan.jpg', 'GongWuYuan10.jpg', 'GongWuYuan11.jpg', 'GongWuYuan12.jpg', 'GongWuYuan13.jpg', 'GongWuYuan14.jpg', 'GongWuYuan15.jpg', 'GongWuYuan16.jpg', 'GongWuYuan3.jpg', 'GongWuYuan4.jpg', 'GongWuYuan5.jpg', 'GongWuYuan6.jpg', 'GongWuYuan7.jpg', 'GongWuYuan8.jpg', 'GongWuYuan9.jpg'),
        '女性': ('NvXing.jpg', 'NvXing10.jpg', 'NvXing11.jpg', 'NvXing12.jpg', 'NvXing13.jpg', 'NvXing14.jpg', 'NvXing15.jpg', 'NvXing2.jpg', 'NvXing3.jpg', 'NvXing4.jpg', 'NvXing5.jpg', 'NvXing6.jpg', 'NvXing7.jpg', 'NvXing8.jpg', 'NvXing9.jpg'),
        '孕前': ('YunQian.jpg', 'YunQian10.jpg', 'YunQian11.jpg', 'YunQian12.jpg', 'YunQian13.jpg', 'YunQian14.jpg', 'YunQian15.jpg', 'YunQian16.jpg', 'YunQian2.jpg', 'YunQian3.jpg', 'YunQian4.jpg', 'YunQian5.jpg', 'YunQian6.jpg', 'YunQian7.jpg', 'YunQian8.jpg', 'YunQian9.jpg'),
        '男女通用': ('tongyong1.jpg', 'tongyong2.jpg', 'tongyong3.jpg', 'tongyong4.jpg', 'tongyong5.jpg'),
        '男性': ('NanXing.jpg', 'NanXing10.jpg', 'NanXing11.jpg', 'NanXing12.jpg', 'NanXing13.jpg', 'NanXing2.jpg', 'NanXing3.jpg', 'NanXing4.jpg', 'NanXing5.jpg', 'NanXing6.jpg', 'NanXing7.jpg', 'NanXing8.jpg', 'NanXing9.jpg'),
        '老年人': ('LaoNianRen.jpg', 'LaoNianRen10.jpg', 'LaoNianRen11.jpg', 'LaoNianRen12.jpg', 'LaoNianRen13.jpg', 'LaoNianRen14.jpg', 'LaoNianRen15.jpg', 'LaoNianRen2.jpg', 'LaoNianRen3.jpg', 'LaoNianRen4.jpg', 'LaoNianRen5.jpg', 'LaoNianRen6.jpg', 'LaoNianRen7.jpg', 'LaoNianRen8.jpg', 'LaoNianRen9.jpg'),
        '青少年': ('QingShaoNian.jpg', 'QingShaoNian10.jpg', 'QingShaoNian11.jpg', 'QingShaoNian12.jpg', 'QingShaoNian13.jpg', 'QingShaoNian14.jpg', 'QingShaoNian15.jpg', 'QingShaoNian2.jpg', 'QingShaoNian3.jpg', 'QingShaoNian4.jpg', 'QingShaoNian5.jpg', 'QingShaoNian6.jpg', 'QingShaoNian7.jpg', 'QingShaoNian8.jpg', 'QingShaoNian9.jpg'),
        '青年': ('QingNian.jpg', 'QingNian10.jpg', 'QingNian11.jpg', 'QingNian12.jpg', 'QingNian13.jpg', 'QingNian14.jpg', 'QingNian15.jpg', 'QingNian3.jpg', 'QingNian4 2.jpg', 'QingNian4.jpg', 'QingNian5.jpg', 'QingNian6.jpg', 'QingNian7.jpg', 'QingNian8.jpg', 'QingNian9.jpg')
    }
    
    def picname_get_url(self, picname):
        return f"http://fm.tijian8.com/fms/package/PACKAGE_IMAGE_INVENTORY/{picname}"
    
    def pictype_get_url(self, 类型, 数量=1):
        return [self.picname_get_url(picname=x) for x in sample(self.图片库[类型], 数量)]
    
    # 字段转小写
    大小写字典 = {chr(i): f"_{chr(i+32)}" for i in range(65, 91)}  # {'A': '_a', 'B': '_b', ... 'Z': '_z'}
    要转换的名称 = {}  # 查表法, key:newkey 对应关系
    不转换的名称 = set()  # 查表法, 不需要转换的key
    def 字段转小写(self, item):
        if type(item) is dict:
            for key in list(item.keys()):
                if key not in self.不转换的名称:
                    if key in self.要转换的名称:
                        item[self.要转换的名称[key]] = item.pop(key)
                    else:
                        new_key = re.sub('^_', '', ''.join([self.大小写字典.get(x, x) for x in key]))
                        if new_key == key:
                            self.不转换的名称 |= {key}
                        else:
                            self.要转换的名称[key] = new_key
                            item[new_key] = item.pop(key)
            for value in item.values():
                self.字段转小写(value)
        elif type(item) is list:
            for x in item:
                self.字段转小写(x)
        return item
    
    # 转中文
    中文字典 = {
        'tag_id': '标签id', 'group_name': '组名称', 'tag_code': '标签代号', 'tag_name': '标签名称', 'group_id': '组id',
        'district_name': '地区名称', 'province_name': '省份名称', 'org_picture': '机构图片', 'org_address': '机构地址',
        'org_id': '机构id', 'org_item_id': '机构单项id', 'org_item_name': '机构单项名称', 'org_item_desc': '机构单项描述',
        'mutual_item_id': '统一单项id', 'mutual_item_name': '统一单项名称', 'mutual_item_desc': '统一单项描述',
        'city_name': '城市名称', 'org_status': '机构状态', 'org_name': '机构名称', 'org_phone': '机构电话',
        'basic': '基础', 'parent_id': '父级id', 'description': '描述', 'genders': '性别', 'tag_group': '标签组',
        'package_id': '套餐id', 'package_name': '套餐名称', 'package_price': '套餐价格', 'package_status': '套餐状态',
        'big_picture': '大图', 'small_picture': '小图', 'sales_volume': '销售量', 'tag_item': '标签项目',
        'create_time': '创建时间', 'update_time': '更新时间', 'tag_gender': '标签性别', 'brand_name': '品牌名称',
        'brand_description': '品牌描述', 'brand_province_id': '品牌省份id', 'brand_province_name': '品牌省份名称',
        'brand_city_id': '品牌城市id', 'brand_city_name': '品牌城市名称', 'brand_district_id': '品牌地区id',
        'brand_district_name': '品牌地区名称', 'brand_address': '品牌地址', 'brand_status': '品牌状态',
        'brand_type': '品牌类型', 'brand_id': '品牌id', 'status':'状态', 'system_item_id': '系统单项id',
        'item_group_id': '单项组id', 'system_item_name': '系统单项名称', 'system_item_alias': '系统单项别名',
        'system_item_desc': '系统单项描述', 'system_item_price': '系统单项价格', 'system_item_discount': '系统单项折扣',
        'system_item_code': '系统单项code', 'gender':'性别', 'item_group_name':'单项组名称', 'images': '图片s',
        'system_items': '系统单项s', 'brand_items': '品牌单项s', 'org_items': '机构单项s', 'scm_package':'供应链套餐',
        'package_alias': '套餐别名', 'package_desc': '套餐描述', 'package_original_price': '套餐原价',
        'package_selling_price': '套餐售价', 'package_discount': '套餐折扣', 'package_gender': '套餐性别',
        'sale_num': '销售量', 'package_code':'套餐code', 'item_desc':'单项描述', 'item_name':'单项名称',
        'item_id':'单项id', 'item_price':'单项价格', 'package_item_id':'套餐单项id', 'type_key': '类型key',
        'dict_type_id': '类型id', 'type_id':'类型id', 'type_key':'类型key', 'org_search_url': '机构搜索url',
        'org_province_id': '机构省份id', 'org_district_name': '机构地区名称', 'org_lgt': '机构经度',
        'org_cooperate_mode': '机构合作模式', 'org_code': '机构code', 'channel_type': '渠道类型',
        'channel_alias': '渠道别名', 'channel_name': '渠道名称', 'org_city_name': '机构城市名称',
        'org_city_id': '机构城市id', 'org_settlement_discount': '机构结算折扣', 'business_hour': '营业时间',
        'org_district_id': '机构地区id', 'org_alias': '机构别名', 'org_description': '机构描述', 'org_lat': '机构纬度',
        'org_type': '机构类型', 'org_province_name': '机构省份名称', 'org_channel_id': '机构渠道id',
        'ah_sync_status': '阿里同步状态', 'bh_sync_status': '百度同步状态', 'th_sync_status': '腾讯同步状态',
        'org_publish': '机构公告', 'org_sketch': '机构简介', 'org_level': '机构等级',
        'org_settlement_advice_discount': '机构建议结算折扣', 'channel_sign_type': '渠道签约类型',
        'org_cash_delivery_support': '机构支持到付', 'package_image_id': '套餐图片id',
        'package_image_path': '套餐图片路径', 'package_image_type': '套餐图片类型'
    }
    def 转中文(self, item, 中文字典=None):
        中文字典 = 中文字典 or self.中文字典
        if type(item) is dict:
            keys = list(item)
            newkeys = [中文字典.get(key, key) for key in keys]
            zip_keys_newkeys = zip(keys, newkeys)
            if len(newkeys) == len(set(newkeys)):
                for key, newkey in zip_keys_newkeys:
                    if key != newkey: item[newkey] = item.pop(key)
            else:
                print('newkeys 冲突')
                for x in [k for k, v in dict(Counter(newkeys)).items() if v > 1]:
                    print(f"{x}: {[a for a, b in zip_keys_newkeys if b == x]}")
                raise Exception('newkeys 冲突')
            for value in item.values():
                if type(value) in [list, dict]:
                    self.转中文(value, 中文字典)
        elif type(item) is list:
            for x in item:
                self.转中文(x, 中文字典)
        return item
    
    def 请求服务器(self, 方法, api, data=None, params=None):
        url = f'{self.domain}/{api}'
        data = json.dumps(data) if data else None
        for n in range(3):
            headers = {'Content-Type': 'application/json', 'Authorization': f"attackFind:{self.token}"}
            if 方法 == 'get':
                r = requests.get(url, data=data, params=params, headers=headers).json()
            elif 方法 == 'post':
                r = requests.post(url, data=data, headers=headers).json()
            elif 方法 == 'put':
                r = requests.put(url, data=data, params=params, headers=headers).json()
            if r['message'] == 'TOKEN生成失败':
                self.更新token()
            else:
                return r
        return r
    
    def post(self, api, data=None, 转中文=True):
        r = self.字段转小写(self.请求服务器(方法='post', api=api, data=data))
        return self.转中文(r) if 转中文 else r

    def get(self, api, params=None, 转中文=True):
        r = self.字段转小写(self.请求服务器(方法='get', api=api, params=params))
        return self.转中文(r) if 转中文 else r
    
    def put(self, api, data=None, 转中文=True):
        r = self.字段转小写(self.请求服务器(方法='put', api=api, data=data))
        return self.转中文(r) if 转中文 else r
    
    def 打印字段(self, data):
        print('\n'.join(f"{x} = ''," for x in set(re.findall(", '([^,]+)': ", str(data)))))
    
    def 获取全部(self, api, params={}):
        params['per'] = 1
        params['per'] = self.get(api, params=params)['total']
        return self.get(api, params=params)['data']
    
    def 构建双通道字典(self, item, 反向=False):
        if 反向: item = {v: k for k, v in item.items()}
        item.update({v: v for v in item.values()})
        print(item)
    
    成功的msg = {
        'OK', '提交的类型名称已存在,请修改后重新提交', '成功', '统一单项名称已存在', 'SUCCESS', 'success',
        '单项类型名称已存在', '系统单项名称已存在'
    }
    
    ''' 已验证的 '''
    
    ''' 已验证 '''
    def 获取一级单项列表(self):
        print('正在获取一级单项列表')
        api = 'system/groups'
        data = self.获取全部(api, params=dict(groupLevel=1))
        self.一级单项字典 = item = {}
        for x in data: item[x['单项组id']] = item[x['单项组名称']] = x
        return data
    
    ''' 已验证 '''
    def 获取二级单项列表(self):
        print('正在获取二级单项列表')
        api = 'system/groups'
        data = self.获取全部(api, params=dict(groupLevel=2))
        self.二级单项字典 = item = {}
        for x in data: item[x['单项组id']] = item[x['单项组名称']] = x
        return data
    
    ''' 已验证 '''
    def 获取系统单项列表(self):
        print('正在获取系统单项列表')
        data = self.获取全部(api='system/system-items')
        self.系统单项字典 = item = {}
        for x in data: item[x['系统单项id']] = item[x['系统单项名称']] = x
        return data
    
    ''' 已验证 '''
    def 新增二级单项(self, 一级单项id, 单项名称, 更新字典=True):
        # 无法和任何一级单项重名, 无法和任何二级单项重名, 可以和统一单项重名
        api = 'system/item-group'
        data = dict(itemGroupName=单项名称, parentId=self.一级单项字典[一级单项id]['单项组id'], sort=101)
        r = self.post(api, data=data)
        msg = r.get('message') or str(r)
        if msg in self.成功的msg:
            if 更新字典: self.获取二级单项列表()
            return True
        else:
            return msg

    ''' 已验证 '''
    def 新增系统单项(self, 二级单项id, 单项名称, 更新字典=True):
        # 可以和二级单项重名, 无法和任何三级单项重名
        api = 'system/system-item'
        data = dict(gender=1, status=0, sort=637, systemItemCode='1',
            itemGroupId = self.二级单项字典[二级单项id]['单项组id'], # 二级单项id
            systemItemName = 单项名称)
        r = self.post(api, data=data)
        msg = r.get('message') or str(r)
        if msg in self.成功的msg:
            if 更新字典: self.获取系统单项列表()
            return True
        else:
            return msg
    
    ''' 已验证 '''
    def 生成标签字典(self):
        print('正在生成标签字典')
        类型s = self.get('system/dict-data/union')['data']
        self.转中文(类型s, {'dict_data_union_v_o_l_v2': '二级标签', 'type_value': '类型名称',
                       'dict_data_id': '标签id','data_key': '标签key', 'data_value': '标签名称'})
        for 类型 in 类型s:
            item = {}
            类型key, 类型名称 = 类型['类型key'], 类型['类型名称']
            for 标签 in 类型['二级标签']:
                标签['类型key'], 标签['类型名称'] = 类型key, 类型名称
                item[标签['标签id']] = item[标签['标签名称']] = 标签
            类型['二级标签'] = item
        self.标签字典 = item = {}
        for 类型 in 类型s: item[类型['类型id']] = item[类型['类型名称']] = 类型
        return item
    
    ''' 已验证 '''
    def 获取品牌列表(self):
        print('正在获取品牌列表')
        data = self.获取全部('system/brands')
        self.品牌字典 = item = {}
        for x in data:
            x['旗下机构'] = {}
            item[x['品牌id']] = item[x['品牌名称']] = x
        return data
    
    ''' 已验证 '''
    def 获取机构列表(self):
        print('正在获取机构列表')
        data = self.获取全部('org/organization/list')
        data = self.转中文(data, {'org_busline': '机构交通线路', 'org_customize_support': '机构支持自定义套餐',
                               'org_notice': '机构预约须知'})
        self.机构字典 = item = {}
        for x in data:
            品牌id, 机构id, 机构名称 = x['品牌id'], x['机构id'], x['机构名称']
            item[机构id] = item[机构名称] = x
            if 品牌id in self.品牌字典:
                品牌旗下机构 = self.品牌字典[品牌id]['旗下机构']
                品牌旗下机构[机构id] = 品牌旗下机构[机构名称] = x
        return data
    
    ''' 已验证 '''
    def 获取套餐详情(self, 套餐id):
        data = self.get('package', params=dict(packageId=套餐id))['data']
        self.转中文(data, {
            'organizations':'挂靠机构', 'attributes':'标签','item_b_o':'单项', 'package_presettle_price':'套餐预估结算价',
            'item_group_lv1_id':'一级单项id','item_group_lv2_id':'二级单项id', 'type_value':'类型名称',
            'data_id':'标签id', 'data_key':'标签key', 'data_value':'标签名称', '套餐性别': '性别和婚否'
        })
        return data
    
    ''' api列表 '''

    def 判断是否请求成功(self, r):
        msg = r.get('message') or str(r)
        return True if msg in self.成功的msg else msg
    

    
    def 判断数据完整性(self, s):
        for i, x in enumerate(s.fillna('')):
            if type(x) is str:
                if not x.replace(' ', ''):
                    print(f"[{s.name}] 第 {i + 2} 行为空值")
                    raise Exception(f"[{s.name}] 第 {i + 2} 行为空值")
        return True
    
    def 从excel提取系统单项(self, path):
        # 确定字段
        字段s = dict(一级 = '一级', 二级 = '二级', 三级 = '三级')
        zd = pd.Series(字段s)
        # 读取excel
        df = pd.read_excel(path)
        for c in (zd.一级, zd.二级, zd.三级): self.判断数据完整性(df[c])
        # 判断统一单项是否重复
        单项s = df[zd.三级]
        if len(单项s) != len(set(单项s)):
            print('以下三级单项重复:')
            print([k for k, v in dict(Counter(单项s)).items() if v > 1])
            raise Exception('三级单项重复')
        # 读取单项
        需创建的二级单项 = []
        需创建的三级单项 = []
        for h in df.to_dict('records'):
            一级, 二级, 三级 = h[zd.一级], h[zd.二级], h[zd.三级]
            if 一级 not in self.一级单项字典: raise Exception(f"一级单项 [{一级}] 不存在")
            if 二级 not in self.二级单项字典:
                需创建的二级单项.append((一级, 二级))
            if 三级 not in self.系统单项字典:
                需创建的三级单项.append((二级, 三级))
        if 需创建的二级单项:
            print('\n将创建以下 二级单项:')
            for x in 需创建的二级单项: print(list(x))
        if 需创建的三级单项:
            print('\n将创建以下 三级单项:')
            for x in 需创建的三级单项: print(list(x))
        return 需创建的二级单项, 需创建的三级单项

    def 机构id_get_套餐列表(self, 机构id, 去除test=False):
        机构id = self.机构字典[机构id]['机构id']
        data = dict(orgId=机构id, per=1)
        r = self.get('packages', params=data)
        data['per'] = r['total']
        r = self.get('packages', params=data)['data']
        if 去除test:
            r = [x for x in r if not [y for y in ('test', '测试') if y in x['套餐名称'].lower()]]
        return r

    def 修改套餐(self, 套餐id, 套餐详情=None, **kvs):
        # kvs: 标签s, 图片s, 单项s, 机构ids, 折扣, 性别和婚否, 套餐名称, 原价, 售价, 预估结算价, 上架
        详情 = 套餐详情 or self.获取套餐详情(套餐id=套餐id)
        套餐 = 详情['供应链套餐']
        单项来源字典 = {'系统库': 1, '品牌库': 2, '机构库': 3, 1: 1, 2: 2, 3: 3}
        性别和婚否字典 = {'男': 1, '女': 2, '已婚女': 3, '未婚女': 4, '通用': 5, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5}
        上架字典 = {0:0, 1:1, True:1, False:0, '是':1, '否':0}

        图片s = kvs.get('图片s')
        if 图片s: 图片s = [dict(packageImagePath=url, packageImageType=type_) for url, type_ in 图片s]
        else: 图片s = [dict(packageImagePath=x['套餐图片路径'], packageImageType=x['套餐图片类型']) for x in 详情['图片s']]

        单项s = kvs.get('单项s')
        if 单项s: 单项s = [dict(itemId=单项id, source=单项来源字典[来源]) for 单项id, 来源 in 单项s]
        else:
            if (详情['单项']['品牌单项s'] or 详情['单项']['机构单项s']):
                raise Exception('修改套餐 方法需要增加对 品牌单项 和 机构单项 的支持')
            else:
                单项s = [dict(itemId=x['单项id'], source=x['source']) for x in 详情['单项']['系统单项s']]
        newdata = dict(
            packageId = 套餐id,
            attributes = [dict(dictDataKey=s['标签key'], dictDataValue=s['标签名称'], dictTypeId=s['类型id']) for s in kvs.get('标签s', 详情['标签'])],
            images = 图片s,
            items = 单项s,
            orgIds = kvs.get('机构ids', [x['机构id'] for x in 详情['挂靠机构']]),
            packageName = kvs.get('套餐名称', 详情['供应链套餐']['套餐名称']),
            packageAlias = kvs.get('套餐名称', 套餐['套餐别名'] or 套餐['套餐名称']),
            packageDiscount = kvs.get('折扣', 套餐['套餐折扣']),
            packageGender = 性别和婚否字典[kvs.get('性别和婚否', 套餐['性别和婚否'])],
            packageOriginalPrice = kvs.get('原价', 套餐['套餐原价']),
            packageSellingPrice = kvs.get('售价', 套餐['套餐售价']),
            packagePresettlePrice = kvs.get('预估结算价', 套餐['套餐预估结算价']),
            saleNum = 套餐['销售量'] or 0,
            status = 上架字典[kvs.get('上架', 套餐['状态'])],
        )
        r = self.put('package', data=newdata)
        msg = r.get('message') or str(r)
        print(msg)
        return True if msg in self.成功的msg else msg
    
    def 新增套餐(self, 标签s, 图片s, 单项s, 机构ids, 折扣, 性别和婚否, 套餐名称, 原价, 售价, 预估结算价, 上架):
        单项来源字典 = {'系统库': 1, '品牌库': 2, '机构库': 3, 1: 1, 2: 2, 3: 3}
        性别和婚否字典 = {'男': 1, '女': 2, '已婚女': 3, '未婚女': 4, '通用': 5, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5}
        上架字典 = {0:0, 1:1, True:1, False:0, '是':1, '否':0}
        data = dict(
            attributes = [dict(dictDataKey=s['标签key'], dictDataValue=s['标签名称'], dictTypeId=s['类型id']) for s in 标签s],
            images = [dict(packageImagePath=url, packageImageType=type_) for url, type_ in 图片s],
            items = [dict(itemId=单项id, source=单项来源字典[来源]) for 单项id, 来源 in 单项s],
            orgIds = 机构ids,
            packageAlias = 套餐名称,
            packageDiscount = 折扣,  # 0-10实数
            packageGender = 性别和婚否字典[性别和婚否],
            packageName = 套餐名称,
            packageOriginalPrice = 原价,  # 单位: 分
            packageSellingPrice = 售价,  # 单位: 分
            packagePresettlePrice = 预估结算价,  # 单位: 分
            saleNum = 0,
            status = 上架字典[上架]
        )
        r = self.post('package', data=data)
        msg = r.get('message') or str(r)
        print(msg)
        return True if msg in self.成功的msg else msg
    
    def 提取唯一值(self, s):
        name = s.name
        s = set(s)
        if len(s) != 1: raise Exception(f"{name} - 值不唯一: {s}")
        return list(s)[0]
    
    def text_get_类型标签(self, text, 规则=None):
        text = text.lower()
        规则 = 规则 or {
            '婚前孕前体检': ['婚', '孕', '优生', '超'],
            '入职体检': ['入职', '招工', '员工'],
            '父母体检': ['父母', '夕阳', '老', '爸妈', '感恩', '银发', '天伦'],
            '白领体检': ['白领', '职场', '商务', '精英', '都市'],
            '儿童体检': ['儿童', '少年', '宝宝', '婴儿'],
            '证件体检': ['证件', '执照'],
            '专项体检': ['专项', '糖尿', '瘦', '胖', '心', '脑', '肿瘤', '免疫', '基因', '镜', '血', '亚健康'],
            '深度体检': ['深度', '贵宾', 'vip'],
            '女性体检': ['女', '妈', '母', '奶奶'],
            '男性体检': ['男', '爸', '父', '爷爷'],
            '特惠体检': ['特惠', '经济', '优惠', '特价']
        }
        return set(k for k, v in 规则.items() for x in v if x in text)
    
    def text_get_单项标签(self, text, 规则=None):
        text = text.lower()
        规则 = 规则 or {
            '外科': ['外科'],
            '肝功能': ['肝功'],
            '糖尿病': ['糖尿'],
            '眼科': ['眼'],
            '基因检测': ['基因'],
            '肿瘤筛查': ['肿瘤和癌'],
            '胃镜': ['胃镜'],
            '肠镜': ['肠镜'],
            '心肌功能': ['心肌'],
            '肾功能': ['肾功'],
            '甲状腺功能': ['甲功']
        }
        return set(k for k, v in 规则.items() for x in v if x in text)
    
    def text_get_性别和婚否(self, text):
        if ('男' in text) and ('女' in text): return '通用'
        elif '男' in text: return '男'
        elif '女' in text:
            if '已婚' in text: return '已婚女'
            elif '未婚' in text: return '未婚女'
            else: return '女'
        else: raise Exception(f"性别和婚否只支持[通用, 男, 女, 未婚女, 已婚女], 不支持[{text}]")
    
    def 从excel提取套餐(self, path, 上架, 智能匹配标签,
        机构名称,唯一识别,套餐名称,单项,原价,预估结算价,套餐折扣,图片,性别和婚否):

        # 构建容易取值的字典, 用 zd.机构名称 取代 zd['机构名称']
        zd = pd.Series(dict(
            机构名称=机构名称, 唯一识别=唯一识别, 套餐名称=套餐名称, 单项=单项, 原价=原价,
            预估结算价=预估结算价, 套餐折扣=套餐折扣, 图片=图片, 性别和婚否=性别和婚否
        ))

        # 读取excel
        df = pd.read_excel(path)
        for x in (zd): self.判断数据完整性(df[x])
        for x in 智能匹配标签:
            for y in x['匹配excel字段']: self.判断数据完整性(df[y])

        # 输入单项名称, 返回单项
        def get_dx(name):
            dx = self.系统单项字典.get(name)
            if dx: return dx['系统单项id']
            print(f"单项不存在: [{name}]")
            raise Exception(f"单项不存在: [{name}]")
        
        # 输入标签名称, 返回标签
        def get_tag(分类名称, 标签名称):
            分类 = self.标签字典.get(分类名称)
            if not 分类:
                print(f"标签类型[{分类名称}]不存在")
                raise Exception(f"标签类型[{分类名称}]不存在")
            标签 = 分类['二级标签'].get(标签名称)
            if not 标签:
                print(f"标签类型[{分类名称}]下不存在[{标签名称}]")
                raise Exception(f"标签类型[{分类名称}]下不存在[{标签名称}]")
            return 标签

        分隔符 = '__分隔符__'
        旧分隔符s = ('|', '[SEP]', '[SEG]')

        # 生成套餐
        机构名称 = self.提取唯一值(s=df[zd.机构名称])
        机构id = self.机构字典[机构名称]['机构id']
        上架 = {0:0, 1:1, True:1, False:0, '是':1, '否':0}[上架]
        套餐s = {}

        for 唯一识别, df in df.groupby(zd.唯一识别):

            # 匹配各类标签
            所有标签 = []
            for 标签块 in 智能匹配标签:
                标签分类 = 标签块['网页后台标签分类']
                text = '_'.join(set(y for x in 标签块['匹配excel字段'] for y in set(df[x])))
                text = text.lower()
                tags = set(k for k, v in 标签块['匹配规则'].items() for x in v if x.lower() in text)
                tags = [get_tag(标签分类, x) for x in tags]
                if (not tags) and (标签块['是否必填'] != '否'):
                    raise Exception(f"{唯一识别} - [{标签分类}]下 - 未匹配到任何标签")
                所有标签 += tags

            图片url = self.pictype_get_url(类型=self.提取唯一值(s=df[zd.图片]), 数量=1)[0]
            # 折扣
            折扣 = self.提取唯一值(s=df[zd.套餐折扣])
            if not 0 <= 折扣 <= 10: raise Exception("套餐折扣范围: 0-10")
            # 原价 和 售价
            原价 = self.提取唯一值(s=df[zd.原价]) * 100
            售价 = 原价 * 折扣 / 10
            # 单项
            单项s = 分隔符.join(set(df[zd.单项]))
            for x in 旧分隔符s: 单项s = 单项s.replace(x, 分隔符)
            单项s = re.sub('^ +', '', 单项s)  # 去除开头空格
            单项s = re.sub(' +$', '', 单项s)  # 去除末尾空格
            单项s = set(re.split(f' *{分隔符} *', 单项s))
            单项s = [(单项id, '系统库') for 单项id in set(get_dx(x) for x in 单项s)]
            # data
            套餐s[唯一识别] = dict(
                机构ids = [机构id],
                套餐名称 = self.提取唯一值(s=df[zd.套餐名称]),
                上架 = 上架,
                原价 = 原价,
                售价 = 售价,
                预估结算价 = self.提取唯一值(s=df[zd.预估结算价]) * 100,
                图片s = [(图片url, type_) for type_ in (1, 2, 3, 4)],
                折扣 = 折扣,
                性别和婚否 = self.text_get_性别和婚否(text=self.提取唯一值(s=df[zd.性别和婚否])),
                单项s = 单项s,
                标签s = 所有标签
            )
        return 套餐s
